/* ====================================================================
 * Copyright (c) 1995-1999 The Apache Group.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Apache Group
 *    for use in the Apache HTTP server project (http://www.apache.org/)."
 *
 * 4. The names "Apache Server" and "Apache Group" must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Apache Group
 *    for use in the Apache HTTP server project (http://www.apache.org/)."
 *
 * THIS SOFTWARE IS PROVIDED BY THE APACHE GROUP ``AS IS'' AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE APACHE GROUP OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Group and was originally based
 * on public domain software written at the National Center for
 * Supercomputing Applications, University of Illinois, Urbana-Champaign.
 * For more information on the Apache Group and the Apache HTTP server
 * project, please see <http://www.apache.org/>.
 *
 */

/*
 * mod_userdir... implement the UserDir command.  Broken away from the
 * Alias stuff for a couple of good and not-so-good reasons:
 *
 * 1) It shows a real minimal working example of how to do something like
 *    this.
 * 2) I know people who are actually interested in changing this *particular*
 *    aspect of server functionality without changing the rest of it.  That's
 *    what this whole modular arrangement is supposed to be good at...
 *
 * Modified by Alexei Kosut to support the following constructs
 * (server running at www.foo.com, request for /~bar/one/two.html)
 *
 * UserDir public_html      -> ~bar/public_html/one/two.html
 * UserDir /usr/web         -> /usr/web/bar/one/two.html
 * UserDir /home/ * /www     -> /home/bar/www/one/two.html
 *  NOTE: theses ^ ^ space only added allow it to work in a comment, ignore
 * UserDir http://x/users   -> (302) http://x/users/bar/one/two.html
 * UserDir http://x/ * /y     -> (302) http://x/bar/y/one/two.html
 *  NOTE: here also ^ ^
 *
 * In addition, you can use multiple entries, to specify alternate
 * user directories (a la Directory Index). For example:
 *
 * UserDir public_html /usr/web http://www.xyz.com/users
 *
 * Modified by Ken Coar to provide for the following:
 *
 * UserDir disable[d] username ...
 * UserDir enable[d] username ...
 *
 * If "disabled" has no other arguments, *all* ~<username> references are
 * disabled, except those explicitly turned on with the "enabled" keyword.
 */

#include "httpd.h"
#include "http_config.h"
#include <stdio.h>
#include <string.h>

module userdir_module;

typedef struct userdir_config {
    int globally_disabled;
    char *userdir;
    table *enabled_users;
    table *disabled_users;
}              userdir_config;

/*
 * Server config for this module: global disablement flag, a list of usernames
 * ineligible for UserDir access, a list of those immune to global (but not
 * explicit) disablement, and the replacement string for all others.
 */

static void *create_userdir_config(pool *p, server_rec *s)
{
    userdir_config
    * newcfg = (userdir_config *) ap_pcalloc(p, sizeof(userdir_config));

    newcfg->globally_disabled = 0;
    newcfg->userdir = DEFAULT_USER_DIR;
    newcfg->enabled_users = ap_make_table(p, 4);
    newcfg->disabled_users = ap_make_table(p, 4);
    return (void *) newcfg;
}

#define O_DEFAULT 0
#define O_ENABLE 1
#define O_DISABLE 2

void get_dir(char *source_data, char *start_point, char *return_value, int width);
int strcmp_just(char *source_name, char *target_name);
char del_char(char *source_data);

static const char *set_user_dir(cmd_parms *cmd, void *dummy, char *arg)
{
    userdir_config
    * s_cfg = (userdir_config *) ap_get_module_config
    (
     cmd->server->module_config,
     &userdir_module
    );
    char *username;
    const char
        *usernames = arg;
    char *kw = ap_getword_conf(cmd->pool, &usernames);
    table *usertable;

    /*
     * Let's do the comparisons once.
     */
    if ((!strcasecmp(kw, "disable")) || (!strcasecmp(kw, "disabled"))) {
        /*
         * If there are no usernames specified, this is a global disable - we
         * need do no more at this point than record the fact.
         */
        if (strlen(usernames) == 0) {
            s_cfg->globally_disabled = 1;
            return NULL;
        }
        usertable = s_cfg->disabled_users;
    }
    else if ((!strcasecmp(kw, "enable")) || (!strcasecmp(kw, "enabled"))) {
        /*
         * The "disable" keyword can stand alone or take a list of names, but
         * the "enable" keyword requires the list.  Whinge if it doesn't have
         * it.
         */
        if (strlen(usernames) == 0) {
            return "UserDir \"enable\" keyword requires a list of usernames";
        }
        usertable = s_cfg->enabled_users;
    }
    else {
        /*
         * If the first (only?) value isn't one of our keywords, just copy
         * the string to the userdir string.
         */
        s_cfg->userdir = ap_pstrdup(cmd->pool, arg);
        return NULL;
    }
    /*
     * Now we just take each word in turn from the command line and add it to
     * the appropriate table.
     */
    while (*usernames) {
        username = ap_getword_conf(cmd->pool, &usernames);
        ap_table_setn(usertable, username, kw);
    }
    return NULL;
}

static const command_rec userdir_cmds[] = {
    {"UserDir", set_user_dir, NULL, RSRC_CONF, RAW_ARGS,
    "the public subdirectory in users' home directories, or 'disabled', or 'disabled username username...', or 'enabled username username...'"},
    {NULL}
};

static int translate_userdir(request_rec *r)
{
    void *server_conf = r->server->module_config;
    const userdir_config *s_cfg =
    (userdir_config *) ap_get_module_config(server_conf, &userdir_module);
    char *name = r->uri;
    const char *userdirs = s_cfg->userdir;
    const char *w, *dname;
    char *redirect;
    char *x = NULL;
    struct stat statbuf;
    char home_user_dir[255];
    char home_server_name[30];
/*
    char home_back_server_name[30];
    char home_server_name_c[30];
    char home_server_num[4];
    char home_back_server_num[4];
*/
    char home_user_dbserver[20];
    char home_user_server[20];
    char home_user_server_c[20];
    char line_buf[255];
    char home_sname[30];
    char *ptr_1;
    char *ptr_2;
    char load_check = 0;
    char check_server = 0, c;
    int count, count2, count3;
    FILE *check_filename;

    /*
     * If the URI doesn't match our basic pattern, we've nothing to do with
     * it.
     */
    if (
        (s_cfg->userdir == NULL) ||
        (name[0] != '/') ||
        (name[1] != '~')
        ) {
        return DECLINED;
    }
    dname = name + 2;
    w = ap_getword(r->pool, &dname, '/');
    /*
     * The 'dname' funny business involves backing it up to capture the '/'
     * delimiting the "/~user" part from the rest of the URL, in case there
     * was one (the case where there wasn't being just "GET /~user HTTP/1.0",
     * for which we don't want to tack on a '/' onto the filename).
     */

    if (dname[-1] == '/') {
        --dname;
    }
    /*
     * If there's no username, it's not for us.  Ignore . and .. as well.
     */
    if (w[0] == '\0' || (w[1] == '.' && (w[2] == '\0' || (w[2] == '.' && w[3] == '\0')))) {
        return DECLINED;
    }
    /*
     * Nor if there's an username but it's in the disabled list.
     */
    if (ap_table_get(s_cfg->disabled_users, w) != NULL) {
        return DECLINED;
    }
    /*
     * If there's a global interdiction on UserDirs, check to see if this
     * name is one of the Blessed.
     */
    if (
        s_cfg->globally_disabled &&
        (ap_table_get(s_cfg->enabled_users, w) == NULL)
        ) {
        return DECLINED;
    }
    /*
     * Special cases all checked, onward to normal substitution processing.
     */

    while (*userdirs) {
        const char *userdir = ap_getword_conf(r->pool, &userdirs);
        char *filename = NULL;

        if (strchr(userdir, '*'))
            x = ap_getword(r->pool, &userdir, '*');

	if (userdir[0] == '\0' || ap_os_is_path_absolute(userdir)) {
            if (x) {
#ifdef HAVE_DRIVE_LETTERS
                /*
                 * Crummy hack. Need to figure out whether we have been
                 * redirected to a URL or to a file on some drive. Since I
                 * know of no protocols that are a single letter, if the : is
                 * the second character, I will assume a file was specified
                 *
                 * Still no good for NETWARE, since : is embedded (sys:/home)
                 */
                if (strchr(x + 2, ':'))
#else
                if (strchr(x, ':'))
#endif /* def HAVE_DRIVE_LETTERS */
		{
                    redirect = ap_pstrcat(r->pool, x, w, userdir, dname, NULL);
                    ap_table_setn(r->headers_out, "Location", redirect);
                    return REDIRECT;
                }
                else
                {
                    /* modified by Mailbook
                    Original :
                    
                    filename = ap_pstrcat(r->pool, x, w, userdir, NULL);
                    */
/*-------------------------------------------------------------------------------------------------*/
				load_check = 0;
				sprintf(line_buf, "/maindisk/member/%s/%s", r->hostname, r->hostname);
				if((check_filename = fopen(line_buf, "r")) != NULL){
					if(fgets(line_buf, 100, check_filename) != NULL){
						if((ptr_1 = strchr(line_buf, '\n')) != NULL){
							ptr_2 = ptr_1;
							ptr_2--;
							if(ptr_2[0] == '\r')
								ptr_2[0] = '\0';
							else
								ptr_1[0] = '\0';
						}
						del_char(line_buf);

						strcpy(home_server_name, line_buf);
						load_check = 1;
					}
					fclose(check_filename);
				}
/*
				load_check = 1;
				if(load_check){
					load_check = 0;
					count = 0;count2 = 0;home_server_name[0] = '\0';
					if(strlen(w) > 1){
						sprintf(line_buf, "%c%c", w[0], w[1]);
						count3 = atoi(line_buf);
						if((check_filename = fopen("/maindisk/environment/mbserver", "r")) != NULL){
							while(!feof(check_filename)){
								if(fgets(line_buf, 100, check_filename) != NULL){
									if((ptr_1 = strchr(line_buf, '\n')) != NULL){
										ptr_2 = ptr_1;
										ptr_2--;
										if(ptr_2[0] == '\r')
											ptr_2[0] = '\0';
										else
											ptr_1[0] = '\0';
									}
								}else
									break;

								del_char(line_buf);
								ptr_1 = line_buf;
								if((ptr_2 = strchr(ptr_1, '\t')) != NULL){
									strncpy(home_server_num, ptr_1, ptr_2 - ptr_1);
									home_server_num[ptr_2 - ptr_1] = '\0';
									ptr_2++;
									strcpy(home_server_name_c, ptr_2);
								}else
									break;

								if(check_server == 0){
									strcpy(home_back_server_name, home_server_name_c);
									check_server = 1;
								}

								if(atoi(home_server_num) == count3){
									strcpy(home_server_name, home_server_name_c);
									check_server = 2;
									load_check = 1;
									break;
								}
							}
							fclose(check_filename);

							if(check_server != 2){
								strcpy(home_server_name, home_back_server_name);
								strcpy(home_server_num, home_back_server_num);
								load_check = 1;
							}

							if(atoi(home_server_num) == 0)
								load_check = 0;

							if(strlen(home_server_name) == 0)
								load_check = 0;
						}
					}
				}
*/
/*
				if(load_check){
					load_check = 0;
					sprintf(line_buf, "/maindisk/environment/%s/mbmail", home_server_name);
					if((check_filename = fopen(line_buf, "r")) != NULL){
						if(fgets(home_sname, 100, check_filename) != NULL){
							if((ptr_1 = strchr(home_sname, '\n')) != NULL){
								ptr_2 = ptr_1;
								ptr_2--;
								if(ptr_2[0] == '\r')
									ptr_2[0] = '\0';
								else
									ptr_1[0] = '\0';
							}
							load_check = 1;
						}
						fclose(check_filename);
					}
				}
*/
				if(load_check){
					load_check = 0;
					get_dir(w, "", home_user_dir, 20);

					sprintf(line_buf, "/maindisk/member/%s%s/%s", home_server_name, home_user_dir, w);
					if((check_filename = fopen(line_buf, "r")) != NULL){
						count2=0;count=0;
						while(!feof(check_filename)){
							c = getc(check_filename);
							if(c == '\t'){
								line_buf[count] = '\0';
								count = 0;
								count2++;

								if(count2 == 4){
									break;
								}else{
									switch(count2){
										case 1:
											break;
										case 2:
											break;
										case 3:
											strcpy(home_user_server, line_buf);
											break;
										default:
											break;
									}
								}
							}else if(c != '\r'){
								line_buf[count] = c;
								count++;
							}else if(c == '\n')
								break;
						}
						fclose(check_filename);

						load_check = 1;
/*
						if((check_filename = fopen("/maindisk/environment/mbsvrlist", "r")) != NULL){
							while(!feof(check_filename)){
								if(fgets(line_buf, 255, check_filename) != NULL){
									if((ptr_1 = strchr(line_buf, '\n')) != NULL){
										ptr_2 = ptr_1;
										ptr_2--;
										if(ptr_2[0] == '\r')
											ptr_2[0] = '\0';
										else
											ptr_1[0] = '\0';
									}
								}else{
									home_user_dbserver[0] = '\0';
									break;
								}

								del_char(line_buf);
								ptr_1 = line_buf;
								if((ptr_2 = strchr(ptr_1, '\t')) != NULL){
									strncpy(home_user_server_c, ptr_1, ptr_2 - ptr_1);
									home_user_server_c[ptr_2 - ptr_1] = '\0';
									ptr_2++;
									strcpy(home_user_dbserver, ptr_2);

									if(!strcmp_just(home_user_server, home_user_server_c))
										break;
								}else{
									home_user_dbserver[0] = '\0';
									break;
								}
							}
							fclose(check_filename);

							if(strlen(home_user_dbserver) != 0){
								if(!strcmp_just(home_user_dbserver, home_sname))
									load_check = 1;
							}
						}
*/
					}
				}

				if(load_check){
					sprintf("/disk%s/%s%s/homepage", home_user_server, home_server_name, home_user_dir);
					filename = ap_pstrcat(r->pool, line_buf, NULL);

					if (filename && (!*userdirs || stat(filename, &statbuf) != -1)) {
						r->filename = ap_pstrcat(r->pool, filename, dname, NULL);

						if (*userdirs && dname[0] == 0)
							r->finfo = statbuf;
						return OK;
					}
				}else{
		            return DECLINED;
/*
					sprintf(line_buf, "/home/bizm/web/cgi-bin/n_homepage_redirect/%s.%s/%s/%s", home_user_dbserver, home_server_name, w, dname);
					filename = ap_pstrcat(r->pool, line_buf, NULL);
					if (filename && !*userdirs || (stat("/home/bizm/web/cgi-bin", &statbuf) != -1)) {
						r->filename = ap_pstrcat(r->pool, filename, NULL);
						if (*userdirs && dname[0] == 0)
							r->finfo = statbuf;
						return OK;
					}
*/
				}
/*-------------------------------------------------------------------------------------------------*/
                }
            }
            else
            {
                /* modified by Mailbook
                Original :
                
                filename = ap_pstrcat(r->pool, userdir, "/", w, NULL);
                */
/*-------------------------------------------------------------------------------------------------*/
				load_check = 0;
				sprintf(line_buf, "/maindisk/member/%s/%s", r->hostname, r->hostname);
				if((check_filename = fopen(line_buf, "r")) != NULL){
					if(fgets(line_buf, 100, check_filename) != NULL){
						if((ptr_1 = strchr(line_buf, '\n')) != NULL){
							ptr_2 = ptr_1;
							ptr_2--;
							if(ptr_2[0] == '\r')
								ptr_2[0] = '\0';
							else
								ptr_1[0] = '\0';
						}
						del_char(line_buf);

						strcpy(home_server_name, line_buf);
						load_check = 1;
					}
					fclose(check_filename);
				}
/*
				load_check = 1;
				if(load_check){
					load_check = 0;
					count = 0;count2 = 0;home_server_name[0] = '\0';
					if(strlen(w) > 1){
						sprintf(line_buf, "%c%c", w[0], w[1]);
						count3 = atoi(line_buf);
						if((check_filename = fopen("/maindisk/environment/mbserver", "r")) != NULL){
							while(!feof(check_filename)){
								if(fgets(line_buf, 100, check_filename) != NULL){
									if((ptr_1 = strchr(line_buf, '\n')) != NULL){
										ptr_2 = ptr_1;
										ptr_2--;
										if(ptr_2[0] == '\r')
											ptr_2[0] = '\0';
										else
											ptr_1[0] = '\0';
									}
								}else
									break;

								del_char(line_buf);
								ptr_1 = line_buf;
								if((ptr_2 = strchr(ptr_1, '\t')) != NULL){
									strncpy(home_server_num, ptr_1, ptr_2 - ptr_1);
									home_server_num[ptr_2 - ptr_1] = '\0';
									ptr_2++;
									strcpy(home_server_name_c, ptr_2);
								}else
									break;

								if(check_server == 0){
									strcpy(home_back_server_name, home_server_name_c);
									check_server = 1;
								}

								if(atoi(home_server_num) == count3){
									strcpy(home_server_name, home_server_name_c);
									check_server = 2;
									load_check = 1;
									break;
								}
							}
							fclose(check_filename);

							if(check_server != 2){
								strcpy(home_server_name, home_back_server_name);
								strcpy(home_server_num, home_back_server_num);
								load_check = 1;
							}

							if(atoi(home_server_num) == 0)
								load_check = 0;

							if(strlen(home_server_name) == 0)
								load_check = 0;
						}
					}
				}
*/
/*
				if(load_check){
					load_check = 0;
					sprintf(line_buf, "/maindisk/environment/%s/mbmail", home_server_name);
					if((check_filename = fopen(line_buf, "r")) != NULL){
						if(fgets(home_sname, 100, check_filename) != NULL){
							if((ptr_1 = strchr(home_sname, '\n')) != NULL){
								ptr_2 = ptr_1;
								ptr_2--;
								if(ptr_2[0] == '\r')
									ptr_2[0] = '\0';
								else
									ptr_1[0] = '\0';
							}
							load_check = 1;
						}
						fclose(check_filename);
					}
				}
*/
				if(load_check){
					load_check = 0;
					get_dir(w, "", home_user_dir, 20);

					sprintf(line_buf, "/maindisk/member/%s%s/%s", home_server_name, home_user_dir, w);
					if((check_filename = fopen(line_buf, "r")) != NULL){
						count2=0;count=0;
						while(!feof(check_filename)){
							c = getc(check_filename);
							if(c == '\t'){
								line_buf[count] = '\0';
								count = 0;
								count2++;

								if(count2 == 4){
									break;
								}else{
									switch(count2){
										case 1:
											break;
										case 2:
											break;
										case 3:
											strcpy(home_user_server, line_buf);
											break;
										default:
											break;
									}
								}
							}else if(c != '\r'){
								line_buf[count] = c;
								count++;
							}else if(c == '\n')
								break;
						}
                        fclose(check_filename);

						load_check = 1;
/*
						if((check_filename = fopen("/maindisk/environment/mbsvrlist", "r")) != NULL){
							while(!feof(check_filename)){
								if(fgets(line_buf, 255, check_filename) != NULL){
									if((ptr_1 = strchr(line_buf, '\n')) != NULL){
										ptr_2 = ptr_1;
										ptr_2--;
										if(ptr_2[0] == '\r')
											ptr_2[0] = '\0';
										else
											ptr_1[0] = '\0';
									}
								}else{
									home_user_dbserver[0] = '\0';
									break;
								}

								del_char(line_buf);
								ptr_1 = line_buf;
								if((ptr_2 = strchr(ptr_1, '\t')) != NULL){
									strncpy(home_user_server_c, ptr_1, ptr_2 - ptr_1);
									home_user_server_c[ptr_2 - ptr_1] = '\0';
									ptr_2++;
									strcpy(home_user_dbserver, ptr_2);

									if(!strcmp_just(home_user_server, home_user_server_c))
										break;
								}else{
									home_user_dbserver[0] = '\0';
									break;
								}
							}
							fclose(check_filename);

							if(strlen(home_user_dbserver) != 0){
								if(!strcmp_just(home_user_dbserver, home_sname))
									load_check = 1;
							}
						}
*/
					}
				}

				if(load_check){
					sprintf("/disk%s/%s%s/homepage", home_user_server, home_server_name, home_user_dir);
					filename = ap_pstrcat(r->pool, line_buf, NULL);

					if (filename && (!*userdirs || stat(filename, &statbuf) != -1)) {
						r->filename = ap_pstrcat(r->pool, filename, dname, NULL);

						if (*userdirs && dname[0] == 0)
							r->finfo = statbuf;
						return OK;
					}
				}else{
		            return DECLINED;
/*
					sprintf(line_buf, "/home/bizm/web/cgi-bin/n_homepage_redirect/%s.%s/%s/%s", home_user_dbserver, home_server_name, w, dname);
					filename = ap_pstrcat(r->pool, line_buf, NULL);

					if (filename && !*userdirs || (stat("/home/bizm/web/cgi-bin", &statbuf) != -1)) {
						r->filename = ap_pstrcat(r->pool, filename, NULL);
						if (*userdirs && dname[0] == 0)
							r->finfo = statbuf;
						return OK;
					}
*/
				}
/*-------------------------------------------------------------------------------------------------*/
			}
        }
        else if (strchr(userdir, ':')) {
            redirect = ap_pstrcat(r->pool, userdir, "/", w, dname, NULL);
            ap_table_setn(r->headers_out, "Location", redirect);
            return REDIRECT;
        }
        else {
#if defined(WIN32) || defined(NETWARE)
            /* Need to figure out home dirs on NT and NetWare */
            return DECLINED;
#else                           /* WIN32 & NetWare */
/*-------------------------------------------------------------------------------------------------*/
				load_check = 0;
				sprintf(line_buf, "/maindisk/member/%s/%s", r->hostname, r->hostname);
				if((check_filename = fopen(line_buf, "r")) != NULL){
					if(fgets(line_buf, 100, check_filename) != NULL){
						if((ptr_1 = strchr(line_buf, '\n')) != NULL){
							ptr_2 = ptr_1;
							ptr_2--;
							if(ptr_2[0] == '\r')
								ptr_2[0] = '\0';
							else
								ptr_1[0] = '\0';
						}
						del_char(line_buf);

						strcpy(home_server_name, line_buf);
						load_check = 1;
					}
					fclose(check_filename);
				}
/*
				load_check = 1;
				if(load_check){
					load_check = 0;
					count = 0;count2 = 0;home_server_name[0] = '\0';
					if(strlen(w) > 1){
						sprintf(line_buf, "%c%c", w[0], w[1]);
						count3 = atoi(line_buf);
						if((check_filename = fopen("/maindisk/environment/mbserver", "r")) != NULL){
							while(!feof(check_filename)){
								if(fgets(line_buf, 100, check_filename) != NULL){
									if((ptr_1 = strchr(line_buf, '\n')) != NULL){
										ptr_2 = ptr_1;
										ptr_2--;
										if(ptr_2[0] == '\r')
											ptr_2[0] = '\0';
										else
											ptr_1[0] = '\0';
									}
								}else
									break;

								del_char(line_buf);
								ptr_1 = line_buf;
								if((ptr_2 = strchr(ptr_1, '\t')) != NULL){
									strncpy(home_server_num, ptr_1, ptr_2 - ptr_1);
									home_server_num[ptr_2 - ptr_1] = '\0';
									ptr_2++;
									strcpy(home_server_name_c, ptr_2);
								}else
									break;

								if(check_server == 0){
									strcpy(home_back_server_name, home_server_name_c);
									check_server = 1;
								}

								if(atoi(home_server_num) == count3){
									strcpy(home_server_name, home_server_name_c);
									check_server = 2;
									load_check = 1;
									break;
								}
							}
							fclose(check_filename);

							if(check_server != 2){
								strcpy(home_server_name, home_back_server_name);
								strcpy(home_server_num, home_back_server_num);
								load_check = 1;
							}

							if(atoi(home_server_num) == 0)
								load_check = 0;

							if(strlen(home_server_name) == 0)
								load_check = 0;
						}
					}
				}
*/
/*
				if(load_check){
					load_check = 0;
					sprintf(line_buf, "/maindisk/environment/%s/mbmail", home_server_name);
					if((check_filename = fopen(line_buf, "r")) != NULL){
						if(fgets(home_sname, 100, check_filename) != NULL){
							if((ptr_1 = strchr(home_sname, '\n')) != NULL){
								ptr_2 = ptr_1;
								ptr_2--;
								if(ptr_2[0] == '\r')
									ptr_2[0] = '\0';
								else
									ptr_1[0] = '\0';
							}
							load_check = 1;
						}
						fclose(check_filename);
					}
				}
*/
				if(load_check){
					load_check = 0;
					get_dir(w, "", home_user_dir, 20);

					sprintf(line_buf, "/maindisk/member/%s%s/%s", home_server_name, home_user_dir, w);
					if((check_filename = fopen(line_buf, "r")) != NULL){
						count2=0;count=0;
						while(!feof(check_filename)){
							c = getc(check_filename);
							if(c == '\t'){
								line_buf[count] = '\0';
								count = 0;
								count2++;

								if(count2 == 4){
									break;
								}else{
									switch(count2){
										case 1:
											break;
										case 2:
											break;
										case 3:
											strcpy(home_user_server, line_buf);
											break;
										default:
											break;
									}
								}
							}else if(c != '\r'){
								line_buf[count] = c;
								count++;
							}else if(c == '\n')
								break;
						}
						fclose(check_filename);

						load_check = 1;
/*
						if((check_filename = fopen("/maindisk/environment/mbsvrlist", "r")) != NULL){
							while(!feof(check_filename)){
								if(fgets(line_buf, 255, check_filename) != NULL){
									if((ptr_1 = strchr(line_buf, '\n')) != NULL){
										ptr_2 = ptr_1;
										ptr_2--;
										if(ptr_2[0] == '\r')
											ptr_2[0] = '\0';
										else
											ptr_1[0] = '\0';
									}
								}else{
									home_user_dbserver[0] = '\0';
									break;
								}

								del_char(line_buf);
								ptr_1 = line_buf;
								if((ptr_2 = strchr(ptr_1, '\t')) != NULL){
									strncpy(home_user_server_c, ptr_1, ptr_2 - ptr_1);
									home_user_server_c[ptr_2 - ptr_1] = '\0';
									ptr_2++;
									strcpy(home_user_dbserver, ptr_2);

									if(!strcmp_just(home_user_server, home_user_server_c))
										break;
								}else{
									home_user_dbserver[0] = '\0';
									break;
								}
							}
							fclose(check_filename);

							if(strlen(home_user_dbserver) != 0){
								if(!strcmp_just(home_user_dbserver, home_sname))
									load_check = 1;
							}
						}
*/
					}
				}

				if(load_check){
					sprintf(line_buf, "/disk%s/%s%s/homepage", home_user_server, home_server_name, home_user_dir);
					filename = ap_pstrcat(r->pool, line_buf, NULL);

					if (filename && (!*userdirs || stat(filename, &statbuf) != -1)) {
						r->filename = ap_pstrcat(r->pool, filename, dname, NULL);

						if (*userdirs && dname[0] == 0)
							r->finfo = statbuf;
						return OK;
					}
				}else{
		            return DECLINED;
/*
					sprintf(line_buf, "/home/bizm/web/cgi-bin/n_homepage_redirect/%s.%s/%s/%s", home_user_dbserver, home_server_name, w, dname);
					filename = ap_pstrcat(r->pool, line_buf, NULL);

					if (filename && !*userdirs || (stat("/home/bizm/web/cgi-bin", &statbuf) != -1)) {
						r->filename = ap_pstrcat(r->pool, filename, NULL);

						if (*userdirs && dname[0] == 0)
							r->finfo = statbuf;
						return OK;
					}
*/
				}
/*-------------------------------------------------------------------------------------------------*/
#endif                          /* WIN32 & NetWare */
        }

        /*
         * Now see if it exists, or we're at the last entry. If we are at the
         * last entry, then use the filename generated (if there is one)
         * anyway, in the hope that some handler might handle it. This can be
         * used, for example, to run a CGI script for the user.
         */

/*-------------------------------------------------------------------------------------
        if (filename && (!*userdirs || stat(filename, &statbuf) != -1)) {
            r->filename = ap_pstrcat(r->pool, filename, dname, NULL);
-------------------------------------------------------------------------------------*/
	    /* when statbuf contains info on r->filename we can save a syscall
	     * by copying it to r->finfo
	     */
/*-------------------------------------------------------------------------------------
	    if (*userdirs && dname[0] == 0)
		r->finfo = statbuf;
          return OK;
        }
-------------------------------------------------------------------------------------*/
    }

    return DECLINED;
}

void get_dir(char *source_data, char *start_point, char *return_value, int width)
{
	int str_length = strlen(source_data), count = 0;
	char line_buf[50]="", copy_data[50];

	return_value[0]='\0';

	strcpy(copy_data, source_data);
	for(count=0; count < width; count++)
		if(count >= str_length)
			copy_data[count] = 1;
	copy_data[count] = '\0';

	strcpy(return_value, start_point);
	for(count = 0; count < width; count++){
		sprintf(line_buf, "/%d", copy_data[count]);
		strcat(return_value, line_buf);
	}

	return;
}

int strcmp_just(char *source_name, char *target_name)
{
	char line_buf[100];
	int str_length = strlen(target_name);

	strncpy(line_buf, source_name, str_length);
	line_buf[str_length] = '\0';

	return strcmp(line_buf, target_name);
}

char del_char(char *source_data)
{
	int count = 0, prev_count = 0, next_count = 0, string_length = strlen(source_data);
	char *ptr;

	while(1){
		if((source_data[count] == ' ') || (source_data[count] == '\t')){
			prev_count++;
		}else
			break;
		count++;
	}

	if(string_length != prev_count){
		count = string_length;
		while(1){
			if((source_data[count-1] == ' ') || (source_data[count-1] == '\t')){
				next_count++;
			}else
				break;
			count--;
		}
	}

	ptr = (char *)malloc(sizeof(char) * (string_length - prev_count - next_count + 1));
	for(count = prev_count; count < string_length - next_count; count++)
		ptr[count - prev_count] = source_data[count];
	ptr[count - prev_count] = '\0';
	strcpy(source_data, ptr);
	free(ptr);

	if(prev_count > 0)
		return 1;
	else
		return 0;
}

module userdir_module = {
    STANDARD_MODULE_STUFF,
    NULL,                       /* initializer */
    NULL,                       /* dir config creater */
    NULL,                       /* dir merger --- default is to override */
    create_userdir_config,      /* server config */
    NULL,                       /* merge server config */
    userdir_cmds,               /* command table */
    NULL,                       /* handlers */
    translate_userdir,          /* filename translation */
    NULL,                       /* check_user_id */
    NULL,                       /* check auth */
    NULL,                       /* check access */
    NULL,                       /* type_checker */
    NULL,                       /* fixups */
    NULL,                       /* logger */
    NULL,                       /* header parser */
    NULL,                       /* child_init */
    NULL,                       /* child_exit */
    NULL                        /* post read-request */
};
